@@ -2,6 +2,7 @@ from django.contrib import admin
 from helpdesk.models import Queue, Ticket, FollowUp, PreSetReply, KBCategory
 from helpdesk.models import EscalationExclusion, EmailTemplate, KBItem
 from helpdesk.models import TicketChange, Attachment, IgnoreEmail
+from helpdesk.models import CustomField
 
 class QueueAdmin(admin.ModelAdmin):
     list_display = ('title', 'slug', 'email_address')
@@ -23,6 +24,9 @@ class FollowUpAdmin(admin.ModelAdmin):
 class KBItemAdmin(admin.ModelAdmin):
     list_display = ('category', 'title', 'last_updated',)
     list_display_links = ('title',)
+   
+class CustomFieldAdmin(admin.ModelAdmin):
+    list_display = ('name', 'label', 'data_type')
 
 admin.site.register(Ticket, TicketAdmin)
 admin.site.register(Queue, QueueAdmin)
@@ -33,3 +37,4 @@ admin.site.register(EmailTemplate)
 admin.site.register(KBCategory)
 admin.site.register(KBItem, KBItemAdmin)
 admin.site.register(IgnoreEmail)
+admin.site.register(CustomField, CustomFieldAdmin)
