@@ -251,7 +251,10 @@ class Ticket(models.Model):
         max_length=200,
         )
 
-    queue = models.ForeignKey(Queue)
+    queue = models.ForeignKey(
+        Queue,
+        verbose_name=_('Queue'),
+        )
 
     created = models.DateTimeField(
         _('Created'),
@@ -278,6 +281,7 @@ class Ticket(models.Model):
         related_name='assigned_to',
         blank=True,
         null=True,
+        verbose_name=_('Assigned to'),
         )
 
     status = models.IntegerField(
@@ -450,10 +454,14 @@ class FollowUp(models.Model):
     although all staff can see them.
     """
 
-    ticket = models.ForeignKey(Ticket)
+    ticket = models.ForeignKey(
+        Ticket,
+        verbose_name=_('Ticket'),
+        )
 
     date = models.DateTimeField(
-        _('Date'),
+        _('Date'), 
+        default = datetime.now()
         )
 
     title = models.CharField(
@@ -481,6 +489,7 @@ class FollowUp(models.Model):
         User,
         blank=True,
         null=True,
+        verbose_name=_('User'),
         )
 
     new_status = models.IntegerField(
@@ -505,7 +514,6 @@ class FollowUp(models.Model):
     def save(self, force_insert=False, force_update=False):
         t = self.ticket
         t.modified = datetime.now()
-        self.date = datetime.now()
         t.save()
         super(FollowUp, self).save(force_insert, force_update)
 
@@ -516,7 +524,10 @@ class TicketChange(models.Model):
     etc) are tracked here for display purposes.
     """
 
-    followup = models.ForeignKey(FollowUp)
+    followup = models.ForeignKey(
+        FollowUp,
+        verbose_name=_('Follow-up'),
+        )
 
     field = models.CharField(
         _('Field'),
@@ -570,7 +581,10 @@ class Attachment(models.Model):
     attachment, or it could be uploaded via the web interface.
     """
 
-    followup = models.ForeignKey(FollowUp)
+    followup = models.ForeignKey(
+        FollowUp,
+        verbose_name=_('Follow-up'),
+        )
 
     file = models.FileField(
         _('File'),
@@ -769,7 +783,10 @@ class KBItem(models.Model):
     An item within the knowledgebase. Very straightforward question/answer
     style system.
     """
-    category = models.ForeignKey(KBCategory)
+    category = models.ForeignKey(
+        KBCategory,
+        verbose_name=_('Category'),
+        )
 
     title = models.CharField(
         _('Title'),
@@ -835,7 +852,10 @@ class SavedSearch(models.Model):
         * All tickets containing the word 'billing'.
          etc...
     """
-    user = models.ForeignKey(User)
+    user = models.ForeignKey(
+        User,
+        verbose_name=_('User'),
+        )
 
     title = models.CharField(
         _('Query Name'),
@@ -913,9 +933,10 @@ def create_usersettings(sender, created_models=[], instance=None, created=False,
     If we end up with users with no UserSettings, then we get horrible
     'DoesNotExist: UserSettings matching query does not exist.' errors.
     """
+    from helpdesk.settings import DEFAULT_USER_SETTINGS
     if sender == User and created:
         # This is a new user, so lets create their settings entry.
-        s, created = UserSettings.objects.get_or_create(user=instance)
+        s, created = UserSettings.objects.get_or_create(user=instance, defaults={'settings': DEFAULT_USER_SETTINGS})
         s.save()
     elif UserSettings in created_models:
         # We just created the UserSettings model, lets create a UserSettings
@@ -926,7 +947,7 @@ def create_usersettings(sender, created_models=[], instance=None, created=False,
             try:
                 s = UserSettings.objects.get(user=u)
             except UserSettings.DoesNotExist:
-                s = UserSettings(user=u)
+                s = UserSettings(user=u, settings=DEFAULT_USER_SETTINGS)
                 s.save()
 
 models.signals.post_syncdb.connect(create_usersettings)
@@ -1016,13 +1037,17 @@ class TicketCC(models.Model):
     an existing system user.
     """
 
-    ticket = models.ForeignKey(Ticket)
+    ticket = models.ForeignKey(
+        Ticket,
+        verbose_name=_('Ticket'),
+        )
 
     user = models.ForeignKey(
         User,
         blank=True,
         null=True,
         help_text=_('User who wishes to receive updates for this ticket.'),
+        verbose_name=_('User'),
         )
 
     email = models.EmailField(
@@ -1060,3 +1085,104 @@ class TicketCC(models.Model):
 
     def __unicode__(self):
         return u'%s for %s' % (self.display, self.ticket.title)
+
+
+class CustomField(models.Model):
+    """
+    Definitions for custom fields that are glued onto each ticket.
+    """
+
+    name = models.SlugField(
+        _('Field Name'),
+        help_text=_('As used in the database and behind the scenes. Must be unique and consist of only lowercase letters with no punctuation.'),
+        unique=True,
+        )
+
+    label = models.CharField(
+        _('Label'),
+        max_length='30',
+        help_text=_('The display label for this field'),
+        )
+
+    help_text = models.TextField(
+        _('Help Text'),
+        help_text=_('Shown to the user when editing the ticket'),
+        blank=True,
+        null=True
+        )
+
+    DATA_TYPE_CHOICES = (
+            ('varchar', _('Character (single line)')),
+            ('text', _('Text (multi-line)')),
+            ('integer', _('Integer')),
+            ('decimal', _('Decimal')),
+            ('list', _('List')),
+            ('boolean', _('Boolean (checkbox yes/no)')),
+            ('date', _('Date')),
+            ('time', _('Time')),
+            ('datetime', _('Date & Time')),
+            ('email', _('E-Mail Address')),
+            ('url', _('URL')),
+            ('ipaddress', _('IP Address')),
+            ('slug', _('Slug')),
+            )
+
+    data_type = models.CharField(
+        _('Data Type'),
+        max_length=100,
+        help_text=_('Allows you to restrict the data entered into this field'),
+        choices=DATA_TYPE_CHOICES,
+        )
+
+    max_length = models.IntegerField(
+        _('Maximum Length (characters)'),
+        blank=True,
+        null=True,
+        )
+
+    decimal_places = models.IntegerField(
+        _('Decimal Places'),
+        help_text=_('Only used for decimal fields'),
+        blank=True,
+        null=True,
+        )
+
+    list_values = models.TextField(
+        _('List Values'),
+        help_text=_('For list fields only. Enter one option per line.'),
+        blank=True,
+        null=True,
+        )
+
+    required = models.BooleanField(
+        _('Required?'),
+        help_text=_('Does the user have to enter a value for this field?'),
+        )
+
+    staff_only = models.BooleanField(
+        _('Staff Only?'),
+        help_text=_('If this is ticked, then the public submission form will NOT show this field'),
+        )
+
+    def __unicode__(self):
+        return '%s' % (self.name)
+
+
+class TicketCustomFieldValue(models.Model):
+    ticket = models.ForeignKey(
+        Ticket,
+        verbose_name=_('Ticket'),
+        )
+
+    field = models.ForeignKey(
+        CustomField,
+        verbose_name=_('Field'),
+        )
+
+    value = models.TextField(blank=True, null=True)
+
+    def __unicode__(self):
+        return '%s / %s' % (self.ticket, self.field)
+
+    class Meta:
+        unique_together = ('ticket', 'field'),
